/*eslint-disable no-undef-expression */
'use strict';

const express = require('express');
const router = express.Router();

const BaseFcn = require("./js/baseFcn.js");
const PrcsFcn = require("./js/prcsFcn.js");
const jwt = require('jsonwebtoken');

const config = require("./config.json");

// -----------------------------  DB Val -----------------------------

// 連接資料庫伺服器
const nano = require('nano')(BaseFcn.createDbUrl(config.database));

// 使用指定資料庫
const DB_page_list = nano.db.use('page_list');
const DB_user_groupt = nano.db.use('user_group');


// -------------------------- Router -------------------------- 

/* 
獲取資源方法
1. 取得 Router
'/SendSubMsg/:Msg'
var Msg = req.params.Msg;

2. 取得 POST 資料
app.use(bodyParser.json()); // 一定要設定，否則無法讀取 Body 值
app.post('/SetSysFcn', function (req, res) {
    var FcnID = req.body.FcnID;

3. 查詢 URI 參數
'/SendSubMsg/?Msg=123'
var Msg = req.query.Msg;
*/

// 驗證 JWT
router.all('*', (req, res, next) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    res.json({
      resStatus: "err",
      Msg: "無授權",
    });
    return;
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    //console.log("TTmatch_Router : " + decoded._id);

    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, body) => { // 取得帳號姓名
      if (isOK) {
        next();
      } else {
        res.json({
          resStatus: "err",
          Msg: "授權錯誤",
        });
      } // else
    });
  });
});

router.get('/', (req, res) => {
  res.send("admin_router");
});

// ------------------ 取得頁面清單 ------------------ 

router.get('/page/list', (req, res) => {
  DB_page_list.list({
    include_docs: true
  }).then((datas) => {
    datas.resStatus = "sus";

    res.json(datas);
  }).catch((err) => {
    res.json({
      resStatus: "err",
      Msg: "取得帳號清單失敗 " + err.message
    });
  });
});

// ------------------ 取得群組清單 ------------------ 

router.get('/group/list', (req, res) => {
  DB_user_groupt.list({
    include_docs: true
  }).then((datas) => {
    datas.resStatus = "sus";

    res.json(datas);
  }).catch((err) => {
    res.json({
      resStatus: "err",
      Msg: "取得群組清單失敗 " + err.message
    });
  });
});

// ------------------ 新增群組清單 ------------------ 

router.post('/group', (req, res) => {
  if (!req.body._id)
    return res.json({
      resStatus: "sus",
      Msg: "ID 必填"
    });

  if (!req.body.GroupName)
    return res.json({
      resStatus: "sus",
      Msg: "GroupName 必填"
    });


  DB_user_groupt.insert(req.body).then(() => {
    res.json({
      resStatus: "sus",
      Msg: "新增群組成功"
    });
  }).catch((err) => {
    res.json({
      resStatus: "err",
      Msg: "新增群組失敗 " + err.message
    });
  });
});

// ------------------ 修改群組 ------------------ 

router.put('/group', (req, res) => {
  if (!req.body._id)
    return res.json({
      resStatus: "sus",
      Msg: "ID 必填"
    });

  if (!req.body.GroupName)
    return res.json({
      resStatus: "sus",
      Msg: "GroupName 必填"
    });


  DB_user_groupt.insert(req.body).then(() => {
    res.json({
      resStatus: "sus",
      Msg: "修改群組成功"
    });
  }).catch((err) => {
    res.json({
      resStatus: "err",
      Msg: "修改群組失敗 " + err.message
    });
  });
});


module.exports = router;


// -------------------------- Function --------------------------