/*globals next */
'use strict';

//const LocalServer = "http://61.219.217.8:4031";

const express = require('express');
const bodyParser = require('body-parser');
const cookieParser = require('cookie-parser');
const app = express();
const SHA256 = require("crypto-js/sha256");
const jwt = require('jsonwebtoken');


const BaseFcn = require("./js/baseFcn.js");
const PrcsFcn = require("./js/prcsFcn.js");

const config = require("./config.json");

// -----------------------------  Value -----------------------------

// 連接資料庫伺服器
const nano = require('nano')(BaseFcn.createDbUrl(config.database));

const DB_account = nano.db.use('account');


// -------------------------- app Setup -------------------------- 

// 轉換 req 資料
app.use(bodyParser.urlencoded({
  extended: true
}));
app.use(bodyParser.json());

// 靜態目錄
app.use(express.static(__dirname + '/public'));

// 設定簽章
app.use(cookieParser(PrcsFcn.Signkey));



// -------------------------- app Router -------------------------- 

const ttmatch_router = require('./ttmatch_router.js');
const admin_router = require('./admin_router.js');

app.use('/TTmatch', ttmatch_router);
app.use('/admin', admin_router);

app.get('/', (req, res) => {
  res.redirect('/login');
});

// -------------------------- Login Logout -------------------------- 

app.get('/login', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    res.sendFile(__dirname + '/html/LoginPage.html', {});
    return;
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, body) => { // 取得帳號姓名
      if (isOK) {
        res.redirect('/index');
      } else {
        res.sendFile(__dirname + '/html/LoginPage.html', {});
      } // else
    });
  });
}).post('/login', (req, res) => {
  let Account = req.body.Account;
  let Password = SHA256(req.body.Password).toString();

  console.log(BaseFcn.getNowTime());
  console.log("\tpost login");
  console.log("Account       : " + Account);

  PrcsFcn.verify_account(Account, Password, (isOK, body) => {
    if (!isOK)
      return res.json({
        resStatus: "err",
        Msg: "帳號密碼錯誤",
      });

    console.log("\nLogin Sus");

    // 設置 JWT
    let TimeOut = 1000 * 60 * 60 * 24;
    let token = jwt.sign(body, PrcsFcn.Signkey, {
      expiresIn: TimeOut
    });

    // cookie 設定
    res.cookie("Authorization", token, {
      path: '/',
      signed: true,
      //secure: true,
      httpOnly: true,
      maxAge: TimeOut
    });

    return res.json({
      resStatus: "sus",
      url: "/index"
    });
  });
}).get('/logout', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    res.json({
      resStatus: "sus",
      url: "/login"
    });
    return;
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    console.log(BaseFcn.getNowTime());
    console.log("get logout");
    console.log("Account  : " + decoded.Account);
    console.log("\nLogout");

    let cookie_setup = {
      path: '/'
    };

    res.clearCookie('Authorization', cookie_setup);

    res.json({
      resStatus: "sus",
      url: "/login"
    });
  });
});

// 驗證 JWT
app.all('*', (req, res, next) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    res.json({
      resStatus: "err",
      Msg: "無授權",
    });
    return;
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    //console.log("TTmatch_Router : " + decoded._id);

    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, body) => { // 取得帳號姓名
      if (isOK) {
        next();
      } else {
        res.json({
          resStatus: "err",
          Msg: "授權錯誤",
        });
      } // else
    });
  });
});

// -------------------------- 帳號存取 -------------------------- 

// 取得帳號資料
app.get('/user', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    res.json({
      resStatus: "sus",
      url: "/login"
    });
    return;
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, resbody) => {
      if (isOK) {
        res.json({
          resStatus: "sus",
          Account: resbody
        });
        return;
      }

      res.json({
        resStatus: "sus",
        url: "/login"
      });
    });
  });
});

// 新增帳號
app.post('/user', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    return res.json({
      resStatus: "err",
      Msg: "無授權"
    });
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, resbody) => {
      if (!isOK) {
        return res.json({
          resStatus: "err",
          Msg: "無授權"
        });
      }

      let NewAccount_Data = req.body;

      NewAccount_Data.Password = SHA256(req.body.Password).toString();

      DB_account.insert(NewAccount_Data).then(() => {
        res.json({
          resStatus: "sus",
          Msg: "新增帳號成功",
        });
      }).catch((err) => {
        res.json({
          resStatus: "err",
          Msg: "新增帳號失敗 " + err.message
        });
      });

    });
  });
});

// 修改密碼
app.put('/user', (req, res) => {
  let Authorization = req.signedCookies.Authorization;
  let NewPassword = SHA256(req.body.NewPassword).toString();

  if (!Authorization) {
    return res.json({
      resStatus: "err",
      Msg: "無授權"
    });
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, resbody) => {
      if (!isOK) {
        return res.json({
          resStatus: "err",
          Msg: "無授權"
        });
      }

      resbody.Password = NewPassword;

      DB_account.insert(resbody).then((body) => {
        res.json({
          resStatus: "sus",
          Msg: "密碼修改成功，請重新登入",
          url: "/login"
        });
      }).catch((err) => {
        res.json({
          resStatus: "err",
          Msg: "密碼修改失敗 " + err.message
        });
      });
    });
  });
});

// 修改帳號資料
app.put('/user_data', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    return res.json({
      resStatus: "err",
      Msg: "無授權"
    });
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, resbody) => {
      if (!isOK) {
        return res.json({
          resStatus: "err",
          Msg: "無授權"
        });
      }

      PrcsFcn.verify_account(req.body._id, "", (isOK, Account_body) => {
        delete req.body.CreationDate;

        if (req.body.Password) // 有密碼，代表管理者修改密碼
          req.body.Password = SHA256(req.body.Password).toString(); // 填入新密碼					
        else
          req.body.Password = Account_body.Password; // 填入原本密碼

        req.body.Date = Account_body.Date;

        DB_account.insert(req.body).then(() => {
          res.json({
            resStatus: "sus",
            Msg: "修改帳號成功",
          });
        }).catch((err) => {
          res.json({
            resStatus: "err",
            Msg: "修改帳號失敗 " + err.message
          });
        });
      });
    });
  });
});

// 刪除帳號
app.delete('/user_data', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    return res.json({
      resStatus: "err",
      Msg: "無授權"
    });
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, resbody) => {
      if (!isOK) {
        return res.json({
          resStatus: "err",
          Msg: "無授權"
        });
      }

      DB_account.destroy(req.body._id, req.body._rev).then(() => {
        res.json({
          resStatus: "sus",
          Msg: "刪除帳號成功",
        });
      }).catch((err) => {
        res.json({
          resStatus: "err",
          Msg: "刪除帳號失敗 " + err.message
        });
      });
    });
  });
});

// 取得帳號清單
app.get('/user/list', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    return res.json({
      resStatus: "err",
      Msg: "無授權"
    });
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, resbody) => {
      if (!isOK) {
        return res.json({
          resStatus: "err",
          Msg: "無授權"
        });
      }

      DB_account.list({
        include_docs: true
      }).then((body) => {
        let Account_List = [];

        body.rows.forEach((Item) => {
          let Item_Doc = Item.doc;

          let CreationDate = BaseFcn.padLeft(Item_Doc.Date[0], 4) + "/" +
            BaseFcn.padLeft(Item_Doc.Date[1], 2) + "/" +
            BaseFcn.padLeft(Item_Doc.Date[2], 2) + " " +
            BaseFcn.padLeft(Item_Doc.Date[3], 2) + ":" +
            BaseFcn.padLeft(Item_Doc.Date[4], 2) + ":" +
            BaseFcn.padLeft(Item_Doc.Date[5], 2);

          let AccountObj = {
            _id: Item_Doc._id,
            _rev: Item_Doc._rev,
            UserName: Item_Doc.UserName,
            Group: Item_Doc.Group,
            CreationDate,
          };

          Account_List.push(AccountObj);
        });

        return res.json({
          resStatus: "sus",
          Account_List
        });
      }).catch((err) => {
        return res.json({
          resStatus: "err",
          Msg: "取得帳號清單失敗 " + err.message
        });
      });
    });
  });
});


// -------------------------- 頁面存取 -------------------------- 

app.get('/index', (req, res) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    res.redirect("/login");
    return;
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
		/*console.log("verify : ");
		console.log(decoded);*/

    if (!decoded._id && !decoded.Password) {
      res.redirect('/login');
      return;
    }


    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, resbody) => {
      if (isOK) {

        res.sendFile(__dirname + '/html/Screen.html', {});
      } else {
        res.redirect('/login');
      }
    });
  });
});


app.get('/:PageName', (req, res, next) => {
  let PageName = req.params.PageName;
  let NameArray = PageName.split(".");

  if (NameArray.length === 2 && NameArray[1] === "html") { // 有 .html 結尾表示請求頁面	
    let Authorization = req.signedCookies.Authorization;

    if (!Authorization) {
      res.send("存取錯誤，請重新登入");
      return;
    }

    jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
      PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, body) => { // 取得帳戶資料
        if (!isOK) { // 重新登入
          res.send("存取錯誤，請重新登入");
          return;
        }

        if (body.AllowPage.indexOf(NameArray[0]) !== -1) { // 是否允許存取頁面
          res.sendFile(__dirname + '/html/' + PageName, {});
          return;
        }

        res.sendFile(__dirname + '/html/NoAuthPage.html', {});

      });
    });
  } else { // 否則為 Router 請求
    console.log("Router : /" + PageName);
    next();
  }
});


process.on('uncaughtException', function (exception) {
  console.error(exception);
});




if (module === require.main) {
  const server = app.listen(process.env.PORT || 8080, () => {
    const port = server.address().port;
    console.log(`App listening on port ${port}`);
  });
}

module.exports = app;