"use strict";

const express = require('express');
const router = express.Router();

const BaseFcn = require("./js/baseFcn.js");
const PrcsFcn = require("./js/prcsFcn.js");
const jwt = require('jsonwebtoken');

const config = require("./config.json");

// -----------------------------  DB Val -----------------------------

// 連接資料庫伺服器
const nano = require('nano')(BaseFcn.createDbUrl(config.database));

// 使用指定資料庫
const ttmatch = nano.db.use('ttmatch_datas');


// -------------------------- Router -------------------------- 

/* 
獲取資源方法
1. 取得 Router
'/SendSubMsg/:Msg'
var Msg = req.params.Msg;

2. 取得 POST 資料
app.use(bodyParser.json()); // 一定要設定，否則無法讀取 Body 值
app.post('/SetSysFcn', function (req, res) {
	var FcnID = req.body.FcnID;

3. 查詢 URI 參數
'/SendSubMsg/?Msg=123'
var Msg = req.query.Msg;
*/

// 驗證 JWT
router.all('*', (req, res, next) => {
  let Authorization = req.signedCookies.Authorization;

  if (!Authorization) {
    res.json({
      resStatus: "err",
      Msg: "無授權",
    });
    return;
  }

  jwt.verify(Authorization, PrcsFcn.Signkey, function (err, decoded) {
    //console.log("TTmatch_Router : " + decoded._id);

    PrcsFcn.verify_account(decoded._id, decoded.Password, (isOK, body) => { // 取得帳號姓名
      if (isOK) {
        next();
      } else {
        res.json({
          resStatus: "err",
          Msg: "授權錯誤",
        });
      } // else
    });
  });
});

router.get('/', (req, res) => {
  res.send("ttmatch_router");
});

// ------------------ 取得戰績 ------------------ 

router.get('/query_log', (req, res) => {

  let Winner = req.query.Winner;
  let Loser = req.query.Loser;
  let Both = req.query.Both;
  let limit = req.query.limit;
  let skip = req.query.skip;
  let StartDate = req.query.StartDate;
  let EndDate = req.query.EndDate;

	/*console.log("Winner    : " + Winner);
	console.log("Loser     : " + Loser);
	console.log("StartDate : " + StartDate);
	console.log("EndDate   : " + EndDate);
	console.log("limit     : " + limit);
	console.log("skip      : " + skip);//*/

  query_log(Winner, Loser, Both, StartDate, EndDate, limit, skip, function (Data) {
    res.json(Data);
  });
});

// ------------------ 取得全部戰績 ------------------ 

router.get('/query_log/info', (req, res) => {
  ttmatch.info().then((body) => {
    res.json(body);
  }).catch((err) => {
    res.json(err);
  });
});


// ------------------ 取得清單 ------------------ 

router.get('/get_list/:MenberType', (req, res) => {
  let MenberType = req.params.MenberType;

  get_winnerList("ttmatch_datas", MenberType, (Datas) => {
    let OriList = Datas.map((Item) => {
      return Item.key;
    });

    let List = {}; // 紀錄人名出現次數，Name: Count
    OriList.forEach(function (item) {
      List[item] = List[item] ? List[item] + 1 : 1;
    });

    let NameArray = Object.keys(List);
    let JSONArray = [];

    NameArray.forEach((Item) => { // 調整格式，方便排序
      JSONArray.push({
        Name: Item,
        Count: List[Item]
      });
    });

    JSONArray.sort(function (a, b) { // 將物件依照 Count 大小排序
      return parseFloat(b.Count) - parseFloat(a.Count);
    });

    let reArr = JSONArray.map((Item) => {
      return Item.Name;
    });

    res.json(reArr);
  });
});


// ------------------ 新增戰績 ------------------ 

router.post('/Data', (req, res) => {
  console.log("post Data");

  let Winner = req.body.Winner;
  let Loser = req.body.Loser;
  let Score1 = req.body.Score1;
  let Score2 = req.body.Score2;
  let Score3 = req.body.Score3;
  let Date = req.body.Date;
  let Remarks = req.body.Remarks;

	/*console.log("Winner  : " + Winner);
	console.log("Loser   : " + Loser);
	console.log("Score1  : " + Score1);
	console.log("Score2  : " + Score2);
	console.log("Score3  : " + Score3);
	console.log("Date    : " + Date);
	console.log("Remarks : " + Remarks);

	console.log("req.body : ");
	console.log(req.body);*/

  let DateArray = [parseInt(Date.split("-")[0], 10), parseInt(Date.split("-")[1], 10), parseInt(Date.split("-")[2], 10)];

  let DataObj = {
    _id: BaseFcn.getUnitTime().toString(16) + BaseFcn.randomChar_length(5),
    winner: Winner,
    loser: Loser,
    Score1,
    Score2,
    Score3,
    Date: DateArray,
    Remarks
  };

  ttmatch.insert(DataObj).then((body) => {
    res.json("sus");
  }).catch((err) => {
    res.json(err);
  });
});

// ------------------ 修改戰績 ------------------ 

router.patch('/Data', (req, res) => {
  let Date = req.body.Date;

  let DateArray = [parseInt(Date.split("/")[0], 10), parseInt(Date.split("/")[1], 10), parseInt(Date.split("/")[2], 10)];

  let DataObj = {
    _id: req.body._id,
    _rev: req.body._rev,
    winner: req.body.Winner,
    loser: req.body.Loser,
    Score1: req.body.Score1,
    Score2: req.body.Score2,
    Score3: req.body.Score3,
    Date: DateArray,
    Remarks: req.body.Remarks
  };

  ttmatch.insert(DataObj).then((body) => {
    res.json("sus");
  }).catch((err) => {
    res.json(err);
  });
});

// ------------------ 刪除戰績 ------------------ 

router.delete('/Data', (req, res) => {
	/*let DataObj = {
		_id: req.body._id,
		_rev: req.body._rev,
	};
	
	//console.log(DataObj);*/

  ttmatch.destroy(req.body._id, req.body._rev).then((body) => {
    res.json("sus");
  }).catch((err) => {
    res.json(err);
  });
});

module.exports = router;


// -------------------------- Function -------------------------- 

function get_winnerList(design, index, callback) {
  ttmatch.view(design, index, {
		/*"limit": 1,
		"descending": true*/
  }, (err, body) => {
    if (err) {
      console.log('[get_ViewByKey err] ' + err.message);
      return;
    }

    callback(body.rows);
  });
}

function query_log(Winner_in, Loser_in, Both_in, SDate_in, EDate_in, limit_in, skip_in, callback) {
  let query = {
    selector: {
      "Date": {
        "$gt": null
      },
			/*"$and": [],
			"$or": []*/
    },
    limit: parseInt(limit_in, 10),
    sort: [{
      "Date": "desc"
    }],
    execution_stats: true
  };
  if (Both_in === "false") {
    if (Winner_in)
      query.selector.winner = {
        $eq: Winner_in
      };

    if (Loser_in)
      query.selector.loser = {
        $eq: Loser_in
      };
  } else {
    query.selector.$or = [];

    let queryObj01 = {};
    let queryObj02 = {};

    if (Winner_in) {
      queryObj01.winner = {
        "$eq": Winner_in
      };
      queryObj02.loser = {
        "$eq": Winner_in
      };
    }
    if (Loser_in) {
      queryObj01.loser = {
        "$eq": Loser_in
      };
      queryObj02.winner = {
        "$eq": Loser_in
      };
    }

    query.selector.$or.push(queryObj01);
    query.selector.$or.push(queryObj02);
  }

  if (SDate_in || EDate_in)
    query.selector.$and = [];


  if (SDate_in) {
    let DateArray = SDate_in.split("-");
    let NumDate = [];
    DateArray.forEach(Item => {
      NumDate.push(parseInt(Item, 10));
    });
    query.selector.$and.push({
      Date: {
        $gte: NumDate
      }
    });
  }

  if (EDate_in) {
    let DateArray = EDate_in.split("-");
    let NumDate = [];
    DateArray.forEach(Item => {
      NumDate.push(parseInt(Item, 10));
    });
    query.selector.$and.push({
      Date: {
        $lte: NumDate
      }
    });
  }

  if (skip_in)
    query.skip = parseInt(skip_in, 10);

  ttmatch.find(query, function (err, body) {
    if (err) {
      console.log("[query ttmatch err] " + err.message);
    } else {
      //console.log(body.docs);
      callback(body);
    }
  }); //*/
}